/*
 * sound.c
 *
 * (C)2000 Dan Potter
 *
 * These are some simple sound interface routines. Most of the bulk of the work
 * is done inside s3mplay, and this part just loads things into the sound RAM and
 * gives our program inside the AICA instructions. 
 *
 * Modified by M.Brent for Cool Herders
 *
 */

#include <stdio.h>
#include <malloc.h>
#include <kallisti/libk.h>
#include <kallisti/timer.h>
#include <kallisti/abi/fs.h>
#include <kallisti/abi/ta.h>
#include <kallisti/abi/spu.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/video.h>
#include <kallisti/abi/thread.h>
#include <kallisti/abi/maple.h>
#include <kallisti/abi/sndsrv_abi.h>
#include <kallisti/irq.h>
#include <kallisti/vmu.h>
#include <kallisti/biosfont.h>
#include "../kosh/abi/conio.h"

#include "sound.h"
#include "sprite.h"
#include "cool.h"

/* AICA firmware bytes */
#include "s3mplay.h"

/* This stuff is pretty hacky - using direct memory access to communicate with the */
/* sound player and so on. But!! It works! :) Thanks Dan :) */

/* SMP interface, right below S3M address */
volatile uint32 *snd_iface = (uint32*)0xa080ffc0;
volatile uint32 *S3M_ADDR_OFFSET = (uint32*)0xa08022d8;
volatile uint32 *S3M_ADDR_PATCH = (uint32*)0xa0800930;
#define S3M_DATA_PATCH 0xea00000c		// change BEQ +0x30 to B +0x30 to skip the sample convert

#define SND_STATUS	0
#define SND_MONO	1
#define SND_SMP		2
#define SND_SMP_WHERE	3
#define SND_SMP_SIZE	4
#define SND_SMP_HZ	5
								

int snd_sheep[6] = {0};
int siz_sheep[6] = {0};
int title_song=0;
int mid_song=0;
int win_song=0;
int no_song=0;

/* Load a bit of data off the disk into sound RAM */
int snd_load(char *fn, int* idx) {
	uint8 buffer[2048];
	int r, oidx = *idx;
	uint32 fd;

	printf("Loading %s at %08x\r\n", fn, *idx);

	fd = fs->open(fn, O_RDONLY);
	if (fd == 0) {
		printf("Can't open file %s\r\n", fn);
		return 0;
	}
	
	/* Load 2048 bytes at a time */
	while ( (r=fs->read(fd, buffer, 2048)) > 0) {
		spu->memload(*idx, buffer, r);
		*idx += r;

	}
	fs->close(fd);

	/* Align on 32-byte boundaries for the AICA chip */
	*idx += 32;
	if (*idx & 0x1f) {
		*idx += 32 - (*idx & 0x1f);
	}
	
	return *idx - oidx;
}

/* Starts playing a pre-loaded music file */
void sound_start(int data, int mono) {
	int idx, i;

	/* Re-initialize the sound */
	spu->disable();

	/* Check mono mode.. if turned on, tell S3MPlay */
	snd_iface[SND_MONO] = mono;
	*(S3M_ADDR_OFFSET)=data;
	*(S3M_ADDR_PATCH)=S3M_DATA_PATCH;
	spu->ram_write_wait();

	/* Start the music playing */
	printf("Starting ARM for %08x\r\n", data);
	spu->enable();

//	/* Wait for program to start (load cycle) */
//	i = thread->jiffies() + 100;
//	while (snd_iface[SND_STATUS] != 3 && thread->jiffies() < i)
//		thread->sleep(10);
//
//	/* Don't need to wait... (end load cycle) */
//	i = thread->jiffies() + 100;
//	while (snd_iface[SND_STATUS] == 3 && thread->jiffies() < i)
//		thread->sleep(10);
}

/* Prepares an S3M by letting the sound player fix up the samples */
void sound_prepare(int data) {
	int idx, i;

	spu->disable();

	LoadProgram();

	*(S3M_ADDR_OFFSET)=data;
	spu->ram_write_wait();

	/* Start the music playing */
	printf("Starting ARM to fix up %08x\r\n", data);
	spu->enable();

	/* Wait for program to start (load cycle) */
	i = thread->jiffies() + 100;
	while (snd_iface[SND_STATUS] != 3 && thread->jiffies() < i)
		thread->sleep(10);

	/* Wait for program to play (end load cycle) */
	i = thread->jiffies() + 200;
	while (snd_iface[SND_STATUS] == 3 && thread->jiffies() < i)
		thread->sleep(10);

	/* we're done. Stop it again */
	spu->disable();
}

/* Play a single sound effect */
void sound_effect(int idx) {
	snd_iface[SND_SMP_WHERE] = snd_sheep[idx];
	snd_iface[SND_SMP_SIZE] = siz_sheep[idx]/2;
	/* Max sample size is 65536, but we have to cut back 32 */
	if (snd_iface[SND_SMP_SIZE] > 65504)
		snd_iface[SND_SMP_SIZE] = 65504;
	snd_iface[SND_SMP_HZ] = 8000;
	snd_iface[SND_SMP] = 1;
	spu->ram_write_wait();
}


/* Initialize the sound system */
/* exits with the SPU disabled */
void sound_init() {
	int idx, idx2;
	char buf[1024];

	/* Start the music playing */
	LoadProgram();

	/* Check mono mode.. if turned on, tell S3MPlay */
	snd_iface[SND_MONO] = 0;

	idx=0x10000;
	// first load the S3Ms, then the samples
	title_song=idx;
	snd_load("title.s3m", &idx);	// title theme
	sound_prepare(title_song);
	mid_song=idx;
	snd_load("mid.s3m", &idx);		// in-game theme
	sound_prepare(mid_song);
	win_song=idx;
	snd_load("end.s3m", &idx);		// game win theme
	sound_prepare(win_song);
	no_song=idx;
	snd_load("none.s3m", &idx);		// silence - to keep the sound player happy
	sound_prepare(no_song);

	for (idx2=0; idx2<6; idx2++) {
		strcpy(buf, "flossie?.raw");// 8khz RAW
		buf[7]=idx2+'1';
		snd_sheep[idx2]=idx;
		siz_sheep[idx2]=snd_load(buf, &idx);
	}
}

/* shutdown */
void sound_shutdown() {
	spu->disable();
}

/* load ARM program */
void LoadProgram() {
	printf("Loading ARM\r\n");
	spu->disable();
	spu->memset(0, 0, sizeof(s3mplay));
	spu->memload(0, s3mplay, sizeof(s3mplay));
}
